import { O as OptionalHasher } from '../shared/gpds.6f120ef4.mjs';
import { CompositeKeyMap } from '../maps/CompositeKeyMap.mjs';
import '../maps/BidirectionalMap.mjs';

var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var _storage, _nodes, _directed, _nodeHasher, _edgeHasher;
class ValuedGraph {
  constructor(options = {}) {
    __privateAdd(this, _storage, new CompositeKeyMap());
    __privateAdd(this, _nodes, /* @__PURE__ */ new Set());
    __privateAdd(this, _directed, void 0);
    __privateAdd(this, _nodeHasher, void 0);
    __privateAdd(this, _edgeHasher, void 0);
    __privateSet(this, _directed, options.directed ?? false);
    __privateSet(this, _nodeHasher, new OptionalHasher(options.hashNode ?? null));
    __privateSet(this, _edgeHasher, new OptionalHasher(options.hashEdge ?? null));
  }
  get directed() {
    return __privateGet(this, _directed);
  }
  add(start, end, edge) {
    const mappedStart = __privateGet(this, _nodeHasher).hash(start);
    const mappedEnd = __privateGet(this, _nodeHasher).hash(end);
    const mappedEdge = __privateGet(this, _edgeHasher).hash(edge);
    __privateGet(this, _nodes).add(mappedStart);
    __privateGet(this, _nodes).add(mappedEnd);
    __privateGet(this, _storage).set([mappedStart, mappedEnd], mappedEdge);
    if (!__privateGet(this, _directed)) {
      __privateGet(this, _storage).set([mappedEnd, mappedStart], mappedEdge);
    }
  }
  delete(start, end) {
    const mappedStart = __privateGet(this, _nodeHasher).hash(start);
    const mappedEnd = __privateGet(this, _nodeHasher).hash(end);
    __privateGet(this, _nodes).delete(mappedStart);
    __privateGet(this, _nodes).delete(mappedEnd);
    __privateGet(this, _storage).delete([mappedStart, mappedEnd]);
    if (!__privateGet(this, _directed)) {
      __privateGet(this, _storage).delete([mappedEnd, mappedStart]);
    }
  }
  getEdge(start, end) {
    const result = __privateGet(this, _storage).get([
      __privateGet(this, _nodeHasher).hash(start),
      __privateGet(this, _nodeHasher).hash(end)
    ]);
    return result ? __privateGet(this, _edgeHasher).unhash(result) : void 0;
  }
  *nodes() {
    for (const node of __privateGet(this, _nodes)) {
      yield __privateGet(this, _nodeHasher).unhash(node);
    }
  }
  *neighborsOf(node) {
    for (const [[neighbor], edge] of __privateGet(this, _storage).entries([
      __privateGet(this, _nodeHasher).hash(node)
    ])) {
      yield {
        node: __privateGet(this, _nodeHasher).unhash(neighbor),
        edge: __privateGet(this, _edgeHasher).unhash(edge)
      };
    }
  }
  // *traverse(
  //   from?: NodeData,
  // ): IterableIterator<{ start: NodeData; end: NodeData; edge: EdgeData }> {
  //   const initial: NodeData | HashResult | undefined =
  //     from !== undefined
  //       ? this.#nodeHasher.hash(from)
  //       : this.#nodes.values().next().value
  //   if (initial === undefined && !this.#nodes.has(undefined)) {
  //     return
  //   }
  //   const visited = new Set<NodeData | HashResult>()
  //   const queue = [initial]
  //   for (const start of queue) {
  //     visited.add(start)
  //     for (const [[end], edge] of this.#storage.entries([start])) {
  //       if (!visited.has(end)) {
  //         queue.push(end)
  //         yield {
  //           start: this.#nodeHasher.unhash(start),
  //           end: this.#nodeHasher.unhash(end),
  //           edge: this.#edgeHasher.unhash(edge),
  //         }
  //       }
  //     }
  //   }
  // }
}
_storage = new WeakMap();
_nodes = new WeakMap();
_directed = new WeakMap();
_nodeHasher = new WeakMap();
_edgeHasher = new WeakMap();

export { ValuedGraph };
