import { O as OptionalHasher } from '../shared/gpds.6f120ef4.mjs';
import './BidirectionalMap.mjs';

var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var __privateMethod = (obj, member, method) => {
  __accessCheck(obj, member, "access private method");
  return method;
};
var _hasher, _compareKeys, _storage, _traverse, traverse_fn;
class CompositeKeyMap {
  constructor(options = {}) {
    __privateAdd(this, _traverse);
    __privateAdd(this, _hasher, void 0);
    __privateAdd(this, _compareKeys, void 0);
    __privateAdd(this, _storage, [void 0, /* @__PURE__ */ new Map()]);
    __privateSet(this, _hasher, new OptionalHasher(options.hashKey));
    __privateSet(this, _compareKeys, options.compareKeys ?? null);
  }
  set(keys, value) {
    const storage = __privateMethod(this, _traverse, traverse_fn).call(this, keys, true);
    storage[0] = value;
  }
  get(keys) {
    const storage = __privateMethod(this, _traverse, traverse_fn).call(this, keys, false);
    return storage?.[0];
  }
  delete(keys) {
    const storage = __privateMethod(this, _traverse, traverse_fn).call(this, keys, false);
    if (!storage) {
      return false;
    }
    if (storage[0] === void 0) {
      return false;
    }
    storage[0] = void 0;
    return true;
  }
  has(keys) {
    const storage = __privateMethod(this, _traverse, traverse_fn).call(this, keys, false);
    return storage ? storage[0] !== void 0 : false;
  }
  clear() {
    __privateGet(this, _hasher).clear();
    __privateSet(this, _storage, [void 0, /* @__PURE__ */ new Map()]);
  }
  *keys(from = []) {
    const storage = __privateMethod(this, _traverse, traverse_fn).call(this, from, false);
    if (!storage) {
      return;
    }
    const stack = [[storage, []]];
    while (stack.length > 0) {
      const [storage2, keys] = stack.shift();
      for (const [key, next] of storage2[1]) {
        if (next[0] !== void 0) {
          yield keys.concat(key).map((key2) => __privateGet(this, _hasher).unhash(key2));
        }
        stack.push([next, keys.concat(key)]);
      }
    }
  }
  *values(from = []) {
    const storage = __privateMethod(this, _traverse, traverse_fn).call(this, from, false);
    if (!storage) {
      return;
    }
    const stack = [storage];
    while (stack.length > 0) {
      const storage2 = stack.shift();
      if (storage2[0] !== void 0) {
        yield storage2[0];
      }
      for (const [, next] of storage2[1]) {
        stack.push(next);
      }
    }
  }
  *entries(from = []) {
    const storage = __privateMethod(this, _traverse, traverse_fn).call(this, from, false);
    if (!storage) {
      return;
    }
    const stack = [[storage, []]];
    while (stack.length > 0) {
      const [storage2, keys] = stack.shift();
      if (storage2[0] !== void 0) {
        yield [keys.map((key) => __privateGet(this, _hasher).unhash(key)), storage2[0]];
      }
      for (const [key, next] of storage2[1]) {
        stack.push([next, keys.concat(key)]);
      }
    }
  }
  *roots() {
    for (const [key] of __privateGet(this, _storage)[1]) {
      yield __privateGet(this, _hasher).unhash(key);
    }
  }
}
_hasher = new WeakMap();
_compareKeys = new WeakMap();
_storage = new WeakMap();
_traverse = new WeakSet();
traverse_fn = function(keys, create) {
  let result = __privateGet(this, _storage);
  const chain = __privateGet(this, _compareKeys) ? keys.toSorted(__privateGet(this, _compareKeys)) : keys;
  for (let i = 0; i < chain.length; i++) {
    const key = __privateGet(this, _hasher).hash(chain[i]);
    const map = result[1];
    let next = map.get(key);
    if (!next) {
      if (create === false) {
        return void 0;
      }
      next = [void 0, /* @__PURE__ */ new Map()];
      map.set(key, next);
    }
    result = next;
  }
  return result;
};

export { CompositeKeyMap };
