var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var _parent, _index, _depth, _size, _children;
const _Node = class _Node {
  constructor() {
    __privateAdd(this, _parent, null);
    __privateAdd(this, _index, -1);
    __privateAdd(this, _depth, 0);
    __privateAdd(this, _size, 1);
    __privateAdd(this, _children, []);
  }
  /**
   * The parent of the node.
   *
   * If the node is a root node, this will be `null`.
   */
  get parent() {
    return __privateGet(this, _parent);
  }
  /**
   * The depth of the node in the tree.
   *
   * The depth of a root node is `0`.
   */
  get depth() {
    return __privateGet(this, _depth);
  }
  /**
   * The size of the node.
   *
   * The size of a node is the number of nodes in the subtree rooted at the node.
   */
  get size() {
    return __privateGet(this, _size);
  }
  /**
   * An iterator that yields the ancestors of the node.
   *
   * The first ancestor is the parent of the node.
   *
   * @returns An iterator that yields the ancestors of the node.
   */
  *ancestors() {
    if (__privateGet(this, _parent) === null) {
      return;
    }
    let node = __privateGet(this, _parent);
    while (true) {
      yield node;
      if (__privateGet(node, _parent) === null) {
        break;
      }
      node = __privateGet(node, _parent);
    }
  }
  /**
   * An iterator that yields the children of the node.
   *
   * @returns An iterator that yields the children of the node.
   */
  children() {
    return __privateGet(this, _children)[Symbol.iterator]();
  }
  /**
   * An iterator that yields the descendants of the node.
   *
   * @param traverse - The traversal order of the descendants.
   * @returns An iterator that yields the descendants of the node.
   */
  *descendants(traverse = _Node.traversal.depth) {
    const stack = [...__privateGet(this, _children)];
    while (stack.length > 0) {
      const node = stack.shift();
      if (traverse === "depth") {
        stack.unshift(...__privateGet(node, _children));
      } else {
        stack.push(...__privateGet(node, _children));
      }
      yield node;
    }
  }
  /**
   * An iterator that yields the leaves of the node.
   *
   * @returns An iterator that yields the leaves of the node.
   */
  *leaves() {
    for (const node of this.descendants()) {
      if (node.isLeaf) {
        yield node;
      }
    }
  }
  /**
   * The next sibling of the node.
   *
   * If the node is the last child of the parent (or does not have a parent), this will be `null`.
   */
  get nextSibling() {
    if (__privateGet(this, _parent) === null) {
      return null;
    }
    return __privateGet(__privateGet(this, _parent), _children)[__privateGet(this, _index) + 1] ?? null;
  }
  /**
   * The previous sibling of the node.
   *
   * If the node is the first child of the parent (or does not have a parent), this will be `null`.
   */
  get previousSibling() {
    if (__privateGet(this, _parent) === null) {
      return null;
    }
    return __privateGet(__privateGet(this, _parent), _children)[__privateGet(this, _index) - 1] ?? null;
  }
  /**
   * The first child of the node.
   *
   * If the node does not have any children, this will be `null`.
   */
  get firstChild() {
    return __privateGet(this, _children)[0] ?? null;
  }
  /**
   * The last child of the node.
   *
   * If the node does not have any children, this will be `null`.
   */
  get lastChild() {
    return __privateGet(this, _children)[__privateGet(this, _children).length - 1] ?? null;
  }
  /**
   * Appends a child to the node.
   *
   * @param child - The child to append.
   * @returns The appended child.
   */
  appendChild(child) {
    if (__privateGet(child, _parent) !== null) {
      __privateGet(child, _parent).removeChild(child);
    }
    __privateSet(child, _parent, this);
    __privateSet(child, _index, __privateGet(this, _children).length);
    __privateSet(child, _depth, __privateGet(this, _depth) + 1);
    __privateSet(this, _size, __privateGet(this, _size) + __privateGet(child, _size));
    __privateGet(this, _children).push(child);
    return child;
  }
  /**
   * Inserts a child before a node.
   *
   * If the node is `null`, the child will be appended to the node.
   *
   * If the child is already a child of the node, it will be moved to the new position.
   *
   * If the child is a child of another node, it will be removed from that node.
   *
   * If the node is not a child of this node, an error will be thrown.
   *
   * @param child - The child to append.
   * @param before - The node to insert the child before.
   * @returns The appended child.
   */
  insertBefore(child, before) {
    if (before === null) {
      return this.appendChild(child);
    }
    if (__privateGet(before, _parent) !== this) {
      throw new Error("Node is not a child of this node");
    }
    if (__privateGet(child, _parent) !== null) {
      __privateGet(child, _parent).removeChild(child);
    }
    __privateSet(child, _parent, this);
    __privateSet(child, _index, __privateGet(before, _index));
    __privateSet(child, _depth, __privateGet(this, _depth) + 1);
    __privateSet(this, _size, __privateGet(this, _size) + __privateGet(child, _size));
    __privateGet(this, _children).splice(__privateGet(before, _index), 0, child);
    for (let i = __privateGet(child, _index) + 1; i < __privateGet(this, _children).length; i++) {
      __privateSet(__privateGet(this, _children)[i], _index, i);
    }
    return child;
  }
  /**
   * Removes a child from the node.
   *
   * If the child is not a child of the node, an error will be thrown.
   *
   * @param child - The child to remove.
   * @returns The removed child.
   */
  removeChild(child) {
    const index = __privateGet(this, _children).indexOf(child);
    if (index === -1) {
      throw new Error("Node is not a child of this node");
    }
    __privateSet(child, _parent, null);
    __privateSet(child, _index, -1);
    __privateSet(child, _depth, 0);
    __privateSet(this, _size, __privateGet(this, _size) - __privateGet(child, _size));
    __privateGet(this, _children).splice(index, 1);
    for (let i = index; i < __privateGet(this, _children).length; i++) {
      __privateSet(__privateGet(this, _children)[i], _index, i);
    }
    return child;
  }
  /**
   * Detaches the node from the tree.
   *
   * If the node is a root node, this will have no effect.
   *
   * This is the same as calling `removeChild` on the parent of the node.
   *
   * @see Node#removeChild
   */
  detach() {
    if (__privateGet(this, _parent) === null) {
      return;
    }
    __privateGet(this, _parent).removeChild(this);
  }
  /**
   * Checks if the node is an ancestor of this node.
   *
   * @param node - The node to check if it is an ancestor of this node.
   * @returns Whether or not the node is an ancestor of this node.
   */
  isAncestorOf(node) {
    for (const ancestor of node.ancestors()) {
      if (ancestor === this) {
        return true;
      }
    }
    return false;
  }
  /**
   * Checks if the node is a descendant of this node.
   *
   * @param node - The node to check if it is a descendant of this node.
   * @returns Whether or not the node is a descendant of this node.
   */
  isDescendantOf(node) {
    return node.isAncestorOf(this);
  }
  /**
   * Checks if the node is a sibling of this node.
   *
   * @param node - The node to check if it is a sibling of this node.
   * @returns Whether or not the node is a sibling of this node.
   */
  isSiblingOf(node) {
    return __privateGet(this, _parent) === __privateGet(node, _parent) && this !== node;
  }
  /**
   * Checks if the node is a leaf node.
   */
  get isLeaf() {
    return __privateGet(this, _children).length === 0;
  }
  /**
   * Checks if the node is a root node.
   */
  get isRoot() {
    return __privateGet(this, _parent) === null;
  }
  /**
   * Checks if the node is the first child of its parent.
   *
   * If the node is a root node, this will be `false`.
   */
  get isFirstChild() {
    return __privateGet(this, _parent) !== null && __privateGet(this, _index) === 0;
  }
  /**
   * Checks if the node is the last child of its parent.
   *
   * If the node is a root node, this will be `false`.
   */
  get isLastChild() {
    return __privateGet(this, _parent) !== null && __privateGet(this, _index) === __privateGet(__privateGet(this, _parent), _children).length - 1;
  }
  /**
   * Finds the common ancestor of the node and another node.
   *
   * @param node - The other node.
   * @returns The common ancestor of the node and the other node, or `null` if there is no common ancestor.
   */
  static commonAncestor(a, b) {
    if (b === a) {
      return b;
    }
    if (b.isAncestorOf(a)) {
      return b;
    }
    if (a.isAncestorOf(b)) {
      return a;
    }
    const deeper = __privateGet(b, _depth) > __privateGet(a, _depth) ? b : a;
    const shallower = __privateGet(b, _depth) > __privateGet(a, _depth) ? a : b;
    const potentialAncestors = new Set(shallower.ancestors());
    for (const ancestor of deeper.ancestors()) {
      if (potentialAncestors.has(ancestor)) {
        return ancestor;
      }
    }
    return null;
  }
};
_parent = new WeakMap();
_index = new WeakMap();
_depth = new WeakMap();
_size = new WeakMap();
_children = new WeakMap();
__publicField(_Node, "traversal", {
  depth: "depth",
  breadth: "breadth"
});
let Node = _Node;

export { Node };
